@@ -23,6 +23,8 @@
 #include <kservicefactory_p.h>
 #include <ksycoca.h>
 
+using namespace Qt::StringLiterals;
+
 // We need a factory that returns the same KService::Ptr every time it's asked for a given service.
 // Otherwise the changes to the service's serviceTypes by KMimeAssociationsTest have no effect
 class FakeServiceFactory : public KServiceFactory
@@ -59,6 +61,11 @@ private:
     QMap<QString, KService::Ptr> m_cache;
 };
 
+static QString menusDir()
+{
+    return QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1String{"/menus"};
+}
+
 // Helper method for all the trader tests, comes from kmimetypetest.cpp
 static bool offerListHasService(const KService::List &offers, const QString &entryPath, bool expected /* if set, show error if not found */)
 {
@@ -133,6 +140,10 @@ private Q_SLOTS:
         QString cacheDir = QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + QLatin1Char('/');
         QDir(cacheDir).removeRecursively();
 
+        QDir(menusDir()).removeRecursively();
+        QDir(menusDir()).mkpath(QStringLiteral("."));
+        QFile::copy(QFINDTESTDATA("test-applications.menu"), menusDir() + QLatin1String("/applications.menu"));
+
         // Create fake application (associated with text/plain in mimeapps.list)
         fakeTextApplication = m_localApps + QLatin1String{"faketextapplication.desktop"};
         writeAppDesktopFile(fakeTextApplication, QStringList() << QStringLiteral("text/plain"));
@@ -494,6 +505,28 @@ private Q_SLOTS:
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
     }
 
+    void testCorrectFallbackOrder()
+    {
+        // Verify that a "more specific type" is used before a "less-specifc"
+        // application/ecmascript inherits from both application/x-executable and text/plain
+        // In this example we should use both of these first which has an app set compared to the less
+        // specifc application/octet-stream via application/x-executable
+        auto queryOnlyDirect = [](const QString &mimeType) {
+            return KApplicationTrader::queryByMimeType(mimeType, [&mimeType](const KService::Ptr &service) {
+                return service->mimeTypes().contains(mimeType);
+            });
+        };
+        const auto offers = assembleServices(KApplicationTrader::queryByMimeType(u"text/javascript"_s));
+        const auto textOffers = assembleServices(queryOnlyDirect(u"text/plain"_s));
+        const auto appOffers = assembleServices(queryOnlyDirect(u"application/x-executable"_s));
+        auto octetOffers = assembleServices(KApplicationTrader::queryByMimeType(u"application/octet-stream"_s));
+        // Apps that only support octet-stream but not text/plain nor application/x-executable
+        octetOffers.removeIf([&textOffers, &appOffers](const QString &app) {
+            return textOffers.contains(app) || appOffers.contains(app);
+        });
+        QCOMPARE(offers.mid(offers.count() - octetOffers.size(), octetOffers.size()), octetOffers);
+    }
+
 private:
     typedef QMap<QString /*mimetype*/, QStringList> ExpectedResultsMap;
 
@@ -501,11 +534,7 @@ private:
     {
         // Wait for notifyDatabaseChanged DBus signal
         // (The real KCM code simply does the refresh in a slot, asynchronously)
-#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 80)
-        QSignalSpy spy(KSycoca::self(), qOverload<const QStringList &>(&KSycoca::databaseChanged));
-#else
         QSignalSpy spy(KSycoca::self(), &KSycoca::databaseChanged);
-#endif
 
         KBuildSycoca builder;
         QVERIFY(builder.recreate());
