@@ -8,8 +8,10 @@
 #ifndef KSERVICEACTION_H
 #define KSERVICEACTION_H
 
-#include <kservice_export.h>
 #include <QSharedDataPointer>
+#include <QVariant>
+#include <kservice_export.h>
+#include <kserviceconversioncheck_p.h>
 class QVariant;
 class KServiceActionPrivate;
 class KService;
@@ -28,27 +30,13 @@ typedef QExplicitlySharedDataPointer<KService> KServicePtr;
 class KSERVICE_EXPORT KServiceAction
 {
 public:
-#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 69)
-    /**
-     * Creates a KServiceAction.
-     * Normally you don't have to do this, KService creates the actions
-     * when parsing the .desktop file.
-     * @deprecated Since 5.69, use the 6-args constructor
-     */
-    KSERVICE_DEPRECATED_VERSION_BELATED(5, 71, 5, 69, "Use the 6-args constructor")
-    KServiceAction(const QString &name, const QString &text,
-                   const QString &icon, const QString &exec,
-                   bool noDisplay = false);
-#endif
     /**
      * Creates a KServiceAction.
      * Normally you don't have to do this, KService creates the actions
      * when parsing the .desktop file.
      * @since 5.69
      */
-    KServiceAction(const QString &name, const QString &text,
-                   const QString &icon, const QString &exec,
-                   bool noDisplay, const KServicePtr &service);
+    KServiceAction(const QString &name, const QString &text, const QString &icon, const QString &exec, bool noDisplay, const KServicePtr &service);
     /**
      * @internal
      * Needed for operator>>
@@ -120,16 +108,32 @@ public:
      */
     KServicePtr service() const;
 
+    /**
+     * Returns the requested property.
+     *
+     * @tparam T the type of the requested property
+     * @param name the name of the requested property
+     * @return the property
+     * @since 6.0
+     */
+    template<typename T>
+    T property(const QString &name) const
+    {
+        KServiceConversionCheck::to_QVariant<T>();
+        return property(name, static_cast<QMetaType::Type>(qMetaTypeId<T>())).value<T>();
+    }
+
 private:
     QSharedDataPointer<KServiceActionPrivate> d;
     friend KSERVICE_EXPORT QDataStream &operator>>(QDataStream &str, KServiceAction &act);
     friend KSERVICE_EXPORT QDataStream &operator<<(QDataStream &str, const KServiceAction &act);
     friend class KService;
-    void setService(const KServicePtr &service);
+    KSERVICE_NO_EXPORT void setService(const KServicePtr &service);
+
+    QVariant property(const QString &_name, QMetaType::Type type) const;
 };
 
 KSERVICE_EXPORT QDataStream &operator>>(QDataStream &str, KServiceAction &act);
 KSERVICE_EXPORT QDataStream &operator<<(QDataStream &str, const KServiceAction &act);
 
 #endif /* KSERVICEACTION_H */
-
